#****************************************************************#
#            (c) RnD Center ELVEES, JSC                          #
# This script is used to make obligatory partitions for SD card, #
# it creates 2 partitions one for boot and the second one for    #
# FTP server storage.                                            #
# Usage example: sudo ./sddisk.sh /dev/sdc                       #
#****************************************************************#

#! /bin/sh

# Helpers function declaration
function printSuccess 
{
    echo "**************************************************"
    echo "*                    SUCESS                      *"
    echo "**************************************************"
}

function printFailure
{
    echo "**************************************************"
    echo "*                    FAILURE                     *"
    echo "**************************************************"
}

function checkResultAndExitIfFailure()
{
    if [ $1 != 0 ]; then
        echo $2
        echo 'Code error: ' $1
        printFailure
        exit
    fi
}

# Main script
echo ""
echo "**************************************************"
echo "*    Create bootable SD disk                     *"
echo "**************************************************"
if [ `whoami` != root ]; then
    echo Please run this script as root or using sudo
    exit
fi   

DRIVE=$1

if [[ ${DRIVE} == "" ]]; then
    echo |Please input path to SD-card device|
    exit
fi

# Default boot pratition size in MB
SIZE=300

if [[ $2 != "" ]]; then
    SIZE=$2
else
    echo "sddisk.sh :: Boot partition size set as default. To change size use SIZE parameter. Example: sudo ./sddisk DRIVE SIZE"
fi

echo Boot partition size: $SIZE MB

umount ${DRIVE}1

# Zero MBR table
dd if=/dev/zero of=$DRIVE bs=1 count=1024
checkResultAndExitIfFailure $? "sddisk.sh :: 'dd' utility failure"

# Make partitions
# NOTE: size of each sector is 512 Bytes, 
#       so to get number of sectors multiply MB to 2 and 1024
let NSECTORS=$SIZE*2*1024

echo "sddisk.sh :: make partitions"
sfdisk --force $DRIVE -uS << EOF
,$NSECTORS
;
EOF
#checkResultAndExitIfFailure "sddisk.sh :: 'sfdisk' utility failure"

# Format partitions
echo "sddisk.sh :: format"
mkfs.msdos -F 32 -n "BOOT" ${DRIVE}1
checkResultAndExitIfFailure $? "sddisk.sh :: 'mkfs' utility failure"
mkfs.msdos -F 32 -n "SDDISK" ${DRIVE}2
checkResultAndExitIfFailure $? "sddisk.sh :: 'mkfs' utility failure"

# Mount BOOT partition
echo "sddisk.sh :: mkdir BOOT"
mkdir -p /run/media/$USERNAME/BOOT/
checkResultAndExitIfFailure $? "sddisk.sh :: 'mkdir' utility failure"
mount ${DRIVE}1 /run/media/$USERNAME/BOOT/
checkResultAndExitIfFailure $? "sddisk.sh :: 'mount' utility failure"

ENV=u-boot.env
cp $ENV /run/media/$USERNAME/BOOT/
checkResultAndExitIfFailure $? "sddisk.sh :: Sorry, can't copy file: $ENV"

OS_IMAGE=`cat ../CMakeLists.txt | grep 'project' | colrm 1 7 | tr -d '() '`
cp ../build/$OS_IMAGE /run/media/$USERNAME/BOOT/
checkResultAndExitIfFailure $? "sddisk.sh :: Sorry, can't copy file: $OS_IMAGE"

# Check copying u-boot.env file
FILE1_SIZE=`wc -c $ENV | awk '{print $1}'`
checkResultAndExitIfFailure $? "sddisk.sh :: 'wc' utility failure"
FILE2_SIZE=`wc -c /run/media/$USERNAME/BOOT/$ENV | awk '{print $1}'`
checkResultAndExitIfFailure $? "sddisk.sh :: 'wc' utility failure"
if [[ $FILE1_SIZE != $FILE2_SIZE ]]; then
    echo 'sddisk.sh :: Copying failure: ' $ENV 
    exit 
fi

# Check copying distrib file
FILE1_SIZE=`wc -c ../build/$OS_IMAGE | awk '{print $1}'`
checkResultAndExitIfFailure $? "sddisk.sh :: 'wc' utility failure"
FILE2_SIZE=`wc -c /run/media/$USERNAME/BOOT/$OS_IMAGE | awk '{print $1}'`
checkResultAndExitIfFailure $? "sddisk.sh :: 'wc' utility failure"
if [[ $FILE1_SIZE != $FILE2_SIZE ]]; then
    echo 'sddisk.sh :: Copying failure: ' $ENV 
    exit 
fi

umount ${DRIVE}1
sudo rm -r -f /run/media/$USERNAME/BOOT/

# if everything is success
printSuccess



