/*******************************************************************************
*  MCom-02 FreeRTOS port                                                       *
*  Copyright (c) 2017 by RnD Center "ELVEES", JSC                              *
*                                                                              *
*  This file is part of the FreeRTOS distribution.                             *
*                                                                              *
*  FreeRTOS is free software; you can redistribute it and/or modify it under   *
*  the terms of the GNU General Public License (version 2) as published by the *
*  Free Software Foundation >>>> AND MODIFIED BY <<<< the FreeRTOS exception.  *
*******************************************************************************/

#include <stdio.h>
#include <ctype.h>

/* include others */
#include "netsrv.h"
#include "../drivers/sd/sd.h"

/* include FreeRTOS files */
#include "../freertos/source/task.h"
#include "../freertos/source/FreeRTOS_FAT/ff_stdio.h"
#include "../freertos/source/FreeRTOS_FAT/ff_ramdisk.h"
#include "../freertos/source/FreeRTOS_FAT/ff_sddisk.h"
#include "../freertos/source/FreeRTOS_TCP/FreeRTOS_IP.h"
#include "../freertos/source/FreeRTOS_TCP/FreeRTOS_Stream_Buffer.h"
#include "../freertos/source/FreeRTOS_TCP/FreeRTOS_IP_Private.h"

/* Function prototypes */
#if( netsrvUDP_SERVICE_TEST_TASK == 1)
void prvUDPServiceTestTask( void *pvParameters );
#endif
void vApplicationSendPingReply( void *pvArgs );
void prvUDPServiceWorkTask( void *pvParameters );
void prvServerWorkTask( void *pvParameters );
extern void vRegisterTCPCLICommands( void );
extern void vStartSimpleTCPServerTasks( uint16_t usStackSize, UBaseType_t uxPriority );
extern void vStartUDPCommandInterpreterTask( uint16_t usStackSize, uint32_t ulPort, UBaseType_t uxPriority );

/* Variables */
uint8_t ucNetworkState = eNetworkStateDown;
uint8_t ucUDPServiceState = eNetworkStateUDPServiceStop;
QueueHandle_t xPingReplyQueue = NULL;
extern uint8_t ucNetworkState;

#if( netsrvUDP_SERVICE_TEST_TASK == 1)
TaskHandle_t xUDPServiceTestTask1Handle = NULL;
#endif

/* Handle of the task that runs the FTP and HTTP servers. */
TCPServer_t *pxTCPServer = NULL;
TaskHandle_t xUDPServiceTaskHandle = NULL;
TaskHandle_t xServerWorkTaskHandle = NULL;
SemaphoreHandle_t xServerReadySemaphore = NULL;
static SemaphoreHandle_t prvStreamBufferTxMutex = NULL;
static StreamBuffer_t *prvStreamBufferTx = NULL;
static uint8_t *prvTmpBuffer = NULL;
static Socket_t prvUDPLocalSocket = NULL;
static FF_Disk_t *pxRAMDisk;
extern sd_card_t xSDCard;
extern SemaphoreHandle_t xSDReadySemaphore;

/* The default IP and MAC address used by the demo.  The address configuration
defined here will be used if ipconfigUSE_DHCP is 0, or if ipconfigUSE_DHCP is
1 but a DHCP server could not be contacted.  See the online documentation for
more information. */
static const uint8_t ucIPAddress[ 4 ] = { configIP_ADDR0, configIP_ADDR1, configIP_ADDR2, configIP_ADDR3 };
static const uint8_t ucNetMask[ 4 ] = { configNET_MASK0, configNET_MASK1, configNET_MASK2, configNET_MASK3 };
static const uint8_t ucGatewayAddress[ 4 ] = { configGATEWAY_ADDR0, configGATEWAY_ADDR1, configGATEWAY_ADDR2, configGATEWAY_ADDR3 };
static const uint8_t ucDNSServerAddress[ 4 ] = { configDNS_SERVER_ADDR0, configDNS_SERVER_ADDR1, configDNS_SERVER_ADDR2, configDNS_SERVER_ADDR3 };

/* Default MAC address configuration. */
const uint8_t ucMACAddress[ 6 ] = { configMAC_ADDR0, configMAC_ADDR1, configMAC_ADDR2, configMAC_ADDR3, configMAC_ADDR4, configMAC_ADDR5 };

void prvAddRAMDiskToFileSystem( void )
{
extern int vCreateAndVerifyExampleFiles( char *pcMountPath );
extern void vStdioWithCWDTest( const char *pcMountPath );
extern void vMultiTaskStdioWithCWDTest( const char *const pcMountPath, uint16_t usStackSizeWords );
static  uint8_t* ucRAMDisk = 0;

        ucRAMDisk = pvPortMalloc(netsrvRAM_DISK_SECTORS * netsrvRAM_DISK_SECTOR_SIZE);
        if(ucRAMDisk == 0)
        {
            configASSERT(0);
            return;
        }
	/* Create the RAM disk. */
	pxRAMDisk = FF_RAMDiskInit( netsrvRAM_DISK_NAME, ucRAMDisk, netsrvRAM_DISK_SECTORS, netsrvIO_MANAGER_CACHE_SIZE );
	configASSERT( pxRAMDisk );

	/* Print out information on the RAM disk. */
	FF_RAMDiskShowPartition( pxRAMDisk );
        
	/* Create a few example files on the disk.  These are not deleted again. */
	vCreateAndVerifyExampleFiles( netsrvRAM_DISK_NAME );

        vUARTServiceFlushWrite(0);
}
/*-----------------------------------------------------------*/

void prvServerWorkTask( void *pvParameters )
{
/* A structure that defines the servers to be created. */
static const struct xSERVER_CONFIG xServerConfiguration[]  __attribute__ ((aligned (8))) =
{
	#if( netsrvCREATE_HTTP_SERVER == 1 )
		/* Server type,		port number,	backlog, 	root dir. */
		{ eSERVER_HTTP, 	80, 			12, 		configHTTP_ROOT },
	#endif /* netsrvCREATE_HTTP_SERVER */

	#if( netsrvCREATE_FTP_SERVER == 1 )
		/* Server type,		port number,	backlog, 	root dir. */
		{ eSERVER_FTP,  	21, 			12, 		"" }
	#endif /* netsrvCREATE_FTP_SERVER */
};
static const uint32_t prvWAIT_MS = 500;

        if( ucNetworkState == eNetworkStateDown )
        {
            FreeRTOS_printf( ( "\r\nInit TCP IP Stack...\n") );
            FreeRTOS_IPInit( ucIPAddress, ucNetMask, ucGatewayAddress, ucDNSServerAddress, ucMACAddress );
        }

        while( ucNetworkState == eNetworkStateDown )
        {
            vTaskDelay( pdMS_TO_TICKS( prvWAIT_MS ) );
        }

        /* Create sync semaphore. It use to response the CLI server that TCP server ready */
        xServerReadySemaphore  = xSemaphoreCreateBinary();
        if( xServerReadySemaphore == NULL )
        {
            FreeRTOS_printf( ("netsrv :: can't create semaphore\n") );
            return;
        }

	/* Remove compiler warning about unused parameter. */
	( void ) pvParameters;
	( void ) xServerConfiguration;

        /* Add storages to file system */
	#if( ( netsrvCREATE_FTP_SERVER == 1 ) || ( netsrvCREATE_HTTP_SERVER == 1 ) )
	{
            prvAddRAMDiskToFileSystem();

	/* The priority of this task can be raised now the disk has been initialised. */
	vTaskPrioritySet( xServerWorkTaskHandle, netsrvTCP_SERVER_TASK_PRIORITY );
                
	const TickType_t xInitialBlockTime = pdMS_TO_TICKS( 200UL );

		/* Wait until the network is up before creating the servers */
                while( ucNetworkState != eNetworkStateUp )
                {
                    vTaskDelay( pdMS_TO_TICKS(500) );
                }

		/* Create the servers defined by the xServerConfiguration array above. */
		pxTCPServer = FreeRTOS_CreateTCPServer( xServerConfiguration, sizeof( xServerConfiguration ) / sizeof( xServerConfiguration[ 0 ] ) );
		configASSERT( pxTCPServer );

                FreeRTOS_printf(( "server-start :: success\r\n" ));
		for( ;; )
		{
			FreeRTOS_TCPServerWork( pxTCPServer, xInitialBlockTime );
		}
	}
	#else
	{
		/* Neither the HTTP nor the FTP server is being used - nothing for
		this task to do.  Set xServerWorkTaskHandle to NULL so nothing tries
		signalling this task, then delete the task. */
		xServerWorkTaskHandle = NULL;
		vTaskDelete( NULL );
	}
	#endif /* ( ( netsrvCREATE_FTP_SERVER == 1 ) || ( netsrvCREATE_HTTP_SERVER == 1 ) ) */
}
/*-----------------------------------------------------------*/

void vNetsrvStartTCPServer( void )
{
BaseType_t xResult = 0;
    
    if( xServerWorkTaskHandle == NULL )
    {           
        xResult = xTaskCreate( prvServerWorkTask,	
                               "SvrWork", 			
                               netsrvTCP_SERVER_STACK_SIZE, 
                               NULL, 			
                               tskIDLE_PRIORITY + 2, 	
                               &xServerWorkTaskHandle );		
        configASSERT(xResult == pdTRUE);
        
        if( xResult != pdTRUE )
        {
            return;
        }
    }
    else
    {
        vTaskResume( xServerWorkTaskHandle );
    }
}
/*-----------------------------------------------------------*/

void vNetsrvStopTCPServer( void )
{
    if( xServerWorkTaskHandle != NULL )
    {
        vTaskSuspend( xServerWorkTaskHandle );
        FreeRTOS_TCPServerStop();
    }
}
/*-----------------------------------------------------------*/

BaseType_t vStartUDPService( uint32_t xIPDest )
{
static const uint32_t prvWAIT_MS = 500;
static uint32_t xTaskParam = 0;

    xTaskParam = xIPDest;
    if( xIPDest == 0 )
    {
        FreeRTOS_printf( ( "\r\nudp service :: error, invlaid ip_destination\n") );
        return -1;
    }

    if( ucNetworkState == eNetworkStateDown )
    {
        FreeRTOS_printf( ( "\r\nInit TCP IP Stack...\n") );
        FreeRTOS_IPInit( ucIPAddress, ucNetMask, ucGatewayAddress, ucDNSServerAddress, ucMACAddress );
    }
    
    while( ucNetworkState == eNetworkStateDown )
    {
        vTaskDelay( pdMS_TO_TICKS( prvWAIT_MS ) );
    }

    if( xUDPServiceTaskHandle == NULL )
    {           
        xTaskCreate( prvUDPServiceWorkTask,
                     "UDP service task",
                     netsrvUDP_SERVICE_STACK_SIZE,
                     NULL,
                     netsrvUDP_SERVICE_PRIORITY,
                     &xUDPServiceTaskHandle );
        configASSERT( xUDPServiceTaskHandle != NULL );
    }
    else
    {
        vTaskResume( xUDPServiceTaskHandle );
    }
    
    
    #if( netsrvUDP_SERVICE_TEST_TASK == 1)
    {
        if( xUDPServiceTestTask1Handle == NULL )
        {
            /* This task send UDP packets to remote server using netsrv interface */
            BaseType_t xTaskCreateResult = pdFALSE;

            xTaskCreateResult = xTaskCreate( prvUDPServiceTestTask,
                                             "UDP service test task 1",
                                             netsrvUDP_SERVICE_STACK_SIZE,
                                             &xTaskParam,
                                             netsrvUDP_SERVICE_PRIORITY,
                                             &xUDPServiceTestTask1Handle );
            configASSERT( xTaskCreateResult == pdTRUE );
        }
        else
        {
            FreeRTOS_printf( ( "udp-start :: service is already run\n" ) );
            return 0;
        }
    }
    #endif

    return 1;
}
/*-----------------------------------------------------------*/

void vStopUDPService( void )
{
    if( xUDPServiceTestTask1Handle != NULL )
    {
        #if( netsrvUDP_SERVICE_TEST_TASK == 1)
        {
            vTaskDelete( xUDPServiceTestTask1Handle );
            xUDPServiceTestTask1Handle = NULL;
        }
        #endif
        vTaskSuspend( xUDPServiceTaskHandle );
    }
}
/*-----------------------------------------------------------*/

int32_t xNetsrvUDPRecvfrom( void *pvBuffer, uint32_t xBufLen, struct freertos_sockaddr *pxSourceAddr )
{   
int32_t xBytesRead = 0;
uint32_t xSourceAddrLen = sizeof( struct freertos_sockaddr );
    
    xBytesRead = FreeRTOS_recvfrom( prvUDPLocalSocket, pvBuffer, xBufLen, 0, pxSourceAddr, &xSourceAddrLen );

    return xBytesRead;
}
/*-----------------------------------------------------------*/

int32_t xNetsrvUDPSendto( void *pvBuffer, uint32_t xBufLen, struct freertos_sockaddr *pxDestAddr )
{
size_t xFreeSpace = 0;
BaseType_t xSemaphoreResult = pdFALSE;
        
    xFreeSpace = uxStreamBufferGetSpace( prvStreamBufferTx );
    
    if( xFreeSpace < sizeof( xBufLen ) + sizeof( struct freertos_sockaddr ) + xBufLen )
    {
        return 0;
    }
    
    xSemaphoreResult = xSemaphoreTake( prvStreamBufferTxMutex, pdMS_TO_TICKS( netsrtUDP_SERVICE_MUTEX_TAKE_DELAY_MS ) );
    {
        if( xSemaphoreResult == pdFALSE )
        {
            /* Can't obtain resource */
            return -1;
        }
        
        /* message structure: [ source_addr | message_len | pointer_to_message ] */
        uxStreamBufferAdd( prvStreamBufferTx, 0, ( uint8_t * )pxDestAddr, sizeof( struct freertos_sockaddr ) );
        uxStreamBufferAdd( prvStreamBufferTx, 0, ( uint8_t * )&xBufLen, sizeof( xBufLen ) );
        uxStreamBufferAdd( prvStreamBufferTx, 0, pvBuffer, xBufLen );
    }
    xSemaphoreGive( prvStreamBufferTxMutex );
    
    return xBufLen;
}
/*-----------------------------------------------------------*/

void prvUDPServiceWorkTask( void *pvParameters )
{
struct freertos_sockaddr xLocalAddr;
struct freertos_sockaddr xDestAddr;
size_t xStreamBufferSize = sizeof( StreamBuffer_t ) - sizeof( prvStreamBufferTx->ucArray ) + netsrvUDP_SRVERVICE_BUF_SIZE_BYTES + 1;
size_t xMsgLength = 0;
size_t xBytesSend = 0;
size_t xBytesToSend = 0;
size_t xSendResult = 0;

    /* Allocate semaphore */
    prvStreamBufferTxMutex = xSemaphoreCreateMutex();
    configASSERT( prvStreamBufferTxMutex != NULL );

    /* Allocate buffer */
    prvStreamBufferTx = pvPortMalloc( xStreamBufferSize ); 
    prvTmpBuffer = ( uint8_t * )pvPortMalloc( netsrvUDP_SRVERVICE_BUF_SIZE_BYTES + 1 );
    configASSERT( prvStreamBufferTx != NULL );
    configASSERT( prvTmpBuffer != NULL );
    prvStreamBufferTx->LENGTH = netsrvUDP_SRVERVICE_BUF_SIZE_BYTES;
    vStreamBufferClear( prvStreamBufferTx );
    
    /* Create socket */
    xLocalAddr.sin_port = FreeRTOS_htons( netsrvUDP_SERVICE_PORT );
    prvUDPLocalSocket = FreeRTOS_socket( FREERTOS_AF_INET, FREERTOS_SOCK_DGRAM, FREERTOS_IPPROTO_UDP );
    configASSERT( prvUDPLocalSocket != FREERTOS_INVALID_SOCKET ); 
    FreeRTOS_bind( prvUDPLocalSocket, &xLocalAddr, sizeof( xLocalAddr ) );
    
    ucUDPServiceState = eNetworkStateUDPServiceRun;

    for(;;)
    {
        if( uxStreamBufferGetSize( prvStreamBufferTx ) )
        {   
            /* Some messages are in buffer */
            uxStreamBufferGet( prvStreamBufferTx, 0, ( uint8_t * )&xDestAddr, sizeof( struct freertos_sockaddr ), pdFALSE );
            uxStreamBufferGet( prvStreamBufferTx, 0, ( uint8_t * )&xMsgLength, sizeof( xMsgLength ), pdFALSE );
            uxStreamBufferGet( prvStreamBufferTx, 0, ( uint8_t * )prvTmpBuffer, xMsgLength, pdFALSE );
            
            xBytesSend = 0;
            xBytesToSend = xMsgLength;
            
            while( xBytesToSend > 0 )
            {
                if( xBytesToSend > ipMAX_UDP_PAYLOAD_LENGTH )
                {
                    xSendResult = FreeRTOS_sendto( prvUDPLocalSocket, prvTmpBuffer + xBytesSend, ipMAX_UDP_PAYLOAD_LENGTH, 0, &xDestAddr, sizeof( xDestAddr ) );
                }
                else
                {
                    xSendResult = FreeRTOS_sendto( prvUDPLocalSocket, prvTmpBuffer + xBytesSend, xBytesToSend, 0, &xDestAddr, sizeof( xDestAddr ) );
                }
                
                if( xSendResult >= 0 )
                {
                    /* If there were no erros, retransmission isn't essential */
                    xBytesSend += xSendResult;
                    xBytesToSend -= xSendResult;
                }
            }
            
        }
        else
        {
            vTaskDelay( pdMS_TO_TICKS( 50 ) );
        }
    }
    
    vPortFree( prvTmpBuffer );
    vPortFree( prvStreamBufferTx );
    FreeRTOS_closesocket( prvUDPLocalSocket );
    vTaskDelete( NULL );
}
/*-----------------------------------------------------------*/

#if( netsrvUDP_SERVICE_TEST_TASK == 1)
void prvUDPServiceTestTask( void *pvParameters )
{    
int xMessageNumber = 0;    
struct freertos_sockaddr xDestAddr;
char pcMessage[ 512 ];

    xDestAddr.sin_addr =  *((uint32_t *)pvParameters);
    xDestAddr.sin_port = FreeRTOS_htons( 5001 );
    
    while( ucUDPServiceState != eNetworkStateUDPServiceRun )
    {
        vTaskDelay( pdMS_TO_TICKS( 500 ) );
    }
    
    FreeRTOS_printf( ( "udp-service-test-task :: start\n") );
    
    for(;;)
    {
        ++xMessageNumber;
        snprintf( pcMessage, 512, "udp-test-task :: %d\n", xMessageNumber );
        xNetsrvUDPSendto( pcMessage, strlen( pcMessage ), &xDestAddr );
        if( xMessageNumber % 10 == 0 )
        {
            vTaskDelay( pdMS_TO_TICKS( 500 ) );
        }
    }
    vTaskDelete( NULL );
}
#endif
/*-----------------------------------------------------------*/

int Basic_stricmp( const char *pc1, const char *pc2 )
{
int iReturn = 0;

	while( *pc1 != 0x00 )
	{
		if( tolower( ( int ) *pc1 ) != tolower( ( int ) *pc2 ) )
		{
			iReturn = 1;
			break;
		}
		else
		{
			pc1++;
			pc2++;
		}
	}

	return iReturn;
}
/*-----------------------------------------------------------*/

BaseType_t xApplicationDNSQueryHook( const char *pcName )
{
BaseType_t xReturn;

	/* Determine if a name lookup is for this node.  Two names are given
	to this node: that returned by pcApplicationHostnameHook() and that set
	by mainDEVICE_NICK_NAME. */
	if( Basic_stricmp( pcName, pcApplicationHostnameHook() ) == 0 )
	{
		xReturn = pdPASS;
	}
	else if( Basic_stricmp( pcName, configDEVICE_NICK_NAME ) == 0 )
	{
		xReturn = pdPASS;
	}
	else
	{
		xReturn = pdFAIL;
	}

	return xReturn;
}
/*-----------------------------------------------------------*/

/* Called by FreeRTOS+TCP when the network connects or disconnects.  Disconnect
events are only received if implemented in the MAC driver. */
void vApplicationIPNetworkEventHook( eIPCallbackEvent_t eNetworkEvent )
{
static uint8_t ucCmdRegistered = 0;    
uint32_t ulIPAddress, ulNetMask, ulGatewayAddress, ulDNSServerAddress;
char cBuffer[ 16 ];
static BaseType_t xTasksAlreadyCreated = pdFALSE;

	FreeRTOS_printf( ( "vApplicationIPNetworkEventHook: event %ld\n", eNetworkEvent ) );

	/* If the network has just come up...*/
	if( eNetworkEvent == eNetworkUp )
	{
		/* Create the tasks that use the IP stack if they have not already been
		created. */
		if( xTasksAlreadyCreated == pdFALSE )
		{
			/* Tasks that use the TCP/IP stack can be created here. */

			/* Start a new task to fetch logging lines and send them out. */
                        #if( netsrvCREATE_UDP_LOGGING_TASK == 1 )
			{
				vUDPLoggingTaskCreate();
			}
			#endif

			#if( ( netsrvCREATE_FTP_SERVER == 1 ) || ( netsrvCREATE_HTTP_SERVER == 1 ) )
			{
				/* Let the server work task now it can now create the servers. */
                                ucNetworkState = eNetworkStateUp;
			}
			#endif

			#if( netsrvCREATE_TCP_ECHO_CLIENT_TASKS_SINGLE == 1 )
			{
				vStartTCPEchoClientTasks_SingleTasks( netsrvECHO_CLIENT_TASK_STACK_SIZE, netsrvECHO_CLIENT_TASK_PRIORITY );
			}
			#endif

			#if( netsrvCREATE_SIMPLE_TCP_ECHO_SERVER == 1 )
			{
				vStartSimpleTCPServerTasks( netsrvECHO_SERVER_STACK_SIZE, netsrvECHO_SERVER_TASK_PRIORITY );
			}
			#endif

			/* Register example commands with the FreeRTOS+CLI command 
			interpreter via the UDP port specified by the
			mainUDP_CLI_PORT_NUMBER constant.  If the HTTP or FTP servers are
			being created then file system related commands will also be
			registered from the TCP server task. */
			#if( netsrvCREATE_UDP_CLI_TASKS == 1 )
			{
				vStartUDPCommandInterpreterTask( netsrvUDP_CLI_TASK_STACK_SIZE, netsrvUDP_CLI_PORT_NUMBER, netsrvUDP_CLI_TASK_PRIORITY );
			}
			#endif

			xTasksAlreadyCreated = pdTRUE;
		}

                if( xPingReplyQueue == NULL )
                {
                BaseType_t xResult = pdFALSE;
                
                    xPingReplyQueue = xQueueCreate( 3, sizeof( uint32_t ) );
                    configASSERT( xPingReplyQueue != NULL );
                    
                    xResult = xTaskCreate( vApplicationSendPingReply, 
                                           "ping-reply-task",
                                           configMINIMAL_STACK_SIZE,
                                           NULL,
                                           tskIDLE_PRIORITY + 1,
                                           NULL );
                    configASSERT( xResult == pdPASS );
                }
               
                if( ucCmdRegistered == 0 )
                {
                    vRegisterTCPCLICommands();
                    ucCmdRegistered = 1;
                }
                
                /* Print out the network configuration, which may have come from a DHCP
		server. */
		FreeRTOS_GetAddressConfiguration( &ulIPAddress, &ulNetMask, &ulGatewayAddress, &ulDNSServerAddress );
		FreeRTOS_inet_ntoa( ulIPAddress, cBuffer );
		FreeRTOS_printf( ( "IP Address: %s\n", cBuffer ) );

		FreeRTOS_inet_ntoa( ulNetMask, cBuffer );
		FreeRTOS_printf( ( "Subnet Mask: %s\n", cBuffer ) );

		FreeRTOS_inet_ntoa( ulGatewayAddress, cBuffer );
		FreeRTOS_printf( ( "Gateway Address: %s\n", cBuffer ) );

		FreeRTOS_inet_ntoa( ulDNSServerAddress, cBuffer );
		FreeRTOS_printf( ( "DNS Server Address: %s\n", cBuffer ) );
	}
}
/*-----------------------------------------------------------*/

void vApplicationPingReplyHook( ePingReplyStatus_t eStatus, uint32_t ulIdentifier )
{
    switch( eStatus )
    {
        case eSuccess :
            /* A valid ping reply has been received.  Post the sequence number
            on the queue that is read by the vApplicationSendPingReply() function below. 
            Don't wait more than 10ms trying to send the message if it cannot be
            sent immediately because this function is called from the IP stack
            task - blocking in this function will block the IP stack. */
            xQueueSend( xPingReplyQueue, &ulIdentifier, 10 / portTICK_PERIOD_MS );
            break;

        case eInvalidChecksum :
        case eInvalidData :
            /* A reply was received but it was not valid. */
            break;
    }
}
/*-----------------------------------------------------------*/

void vApplicationSendPingReply( void *pvArgs )
{
uint32_t ulReceivedMessage = 0;    
uint8_t pucMessageBuffer[ 32 ] = { 0 };
    
    for(;;)
    {
        xQueueReceive( xPingReplyQueue, &ulReceivedMessage, portMAX_DELAY );
        FreeRTOS_inet_ntoa( ulReceivedMessage, pucMessageBuffer );
        FreeRTOS_printf( ( "ping :: request from %s recieved\n", pucMessageBuffer ) );
    }
    vTaskDelete( NULL );
}
/*-----------------------------------------------------------*/


